@@ -4,12 +4,16 @@
 #' and the number of replications. Results can be saved as temporary files in case of interruptions
 #' and may be restored by rerunning the exact function calls again, provided that the respective temp
 #' file can be found in the working directory. To conserve RAM, temporary objects (such as
-#' generated data across conditions and replications) are discarded.
-#' Supports parallel and cluster computing, global and
-#' local debugging, and is designed to be cross-platform.  For a skeleton version of the work-flow
+#' generated data across conditions and replications) are discarded. For longer simulations, however,
+#' it is recommended to use \code{save = TRUE} and/or \code{save_results = TRUE} to temporarily save the
+#' simulation state and to write results to separate external .rds files, respectively.
+#' Supports parallel and cluster computing, global and local debugging, error handling, and is
+#' designed to be cross-platform.
+#'
+#' For a skeleton version of the work-flow
 #' which may be useful when initially defining a simulation, see \code{\link{SimDesign_functions}}.
 #' Additional examples can be found on the package wiki, located at
-#' \url{https://github.com/philchalmers/SimDesign/wiki}
+#' \url{https://github.com/philchalmers/SimDesign/wiki}.
 #'
 #'
 #' The strategy for organizing the Monte Carlo simulation work-flow is to
@@ -24,23 +28,27 @@
 #'    }
 #'    \item{3)}{Pass the above objects to the \code{runSimulation} function, and define the
 #'       number of replications with the \code{replications} input}
-#'    \item{4)}{Analyze the output from \code{runSimulation}, possibly using techniques from ANOVA
+#'    \item{4)}{Analyze the output from \code{runSimulation}, possibly using ANOVA techniques
 #'      and generating suitable plots and tables}
 #' }
 #'
 #' Two constants for each condition are returned by default:
-#' \code{N_CELL_RUNS} to indicate the number of Monte Carlo runs it took to obtain valid results (this will
-#' be greater than the number of replications requested if, for example, models failed to converge
-#' and had to be re-drawn), and \code{SIM_TIME} to indicate how long (in seconds) it took to complete
-#' all the Monte Carlo replications for each respective condition.
+#' \code{REPLICATIONS} to indicate the number of Monte Carlo replications,
+#' \code{SIM_TIME} to indicate how long (in seconds) it took to complete
+#' all the Monte Carlo replications for each respective condition, and if \code{try_errors = TRUE}
+#' then columns containing the number of replications due to \code{try()} errors where the error messages
+#' represent the names of the columns prefixed with a \code{TRY_ERROR_MESSAGE} string.
 #'
-#' In the event of a computer crash, power outage, etc, if the \code{save_every} command has been
-#' used to save temporary results (along with the logical \code{save = TRUE}) then the original code in
+#' @section Storing and resuming temporary results:
+#'
+#' In the event of a computer crash, power outage, etc, if \code{save = TRUE} was used
+#' then the original code in
 #' the main source file need only be rerun again to resume the simulation.
 #' The saved temp file will be read into the function, and the simulation will continue where it left
 #' off before the simulation was terminated. Upon completion, a data.frame with the simulation
 #' will be returned in the R session and a '.rds' file will be saved to the hard-drive (with the
-#' file name corresponding to the \code{filename} argument below).
+#' file name corresponding to the \code{filename} argument below). To save the complete list of results returned
+#' from \code{\link{analyse}} to unique files use \code{save_results = TRUE}.
 #'
 #' @section Cluster computing:
 #'
@@ -56,11 +64,11 @@
 #'
 #' mpirun -np 16 -H localhost,slave1,slave2 R --slave -f simulation.R
 #'
-#' @section Poor man's cluster computing:
+#' @section Poor man's cluster computing for indedependent nodes:
 #'
-#' In the event that you do not have access to a Beowulf-type cluster, but have multiple personal
-#' computers, then the simulation code can be manually distributed across each computer instead.
-#' This simply requires passing a smaller value to the \code{each} argument on each computer, and later
+#' In the event that you do not have access to a Beowulf-type cluster but have multiple personal
+#' computers, then the simulation code can be manually distributed across each independent computer instead.
+#' This simply requires passing a smaller value to the \code{replications} argument on each computer, and later
 #' aggregating the results using the \code{\link{aggregate_simulations}} function.
 #'
 #' For instance, if you have two computers available and wanted 500 replications you
@@ -84,12 +92,14 @@
 #' @param summarise user-defined summary function to be used after all the replications have completed.
 #'    See \code{\link{summarise}} for details
 #'
-#' @param main (optional) user-defined organization function defining how the simulation should be
-#'    organized. When NULL, the internal function definition is used (and the majority of the time
-#'    this is sufficient). See \code{\link{main}} for further details
-#'
 #' @param replications number of replication to perform per condition (i.e., each row in \code{design})
 #'
+#' @param fixed_design_elements (optional) an object (usually a list) containing fixed design elements
+#'   which can be used across all simulation conditions. This is useful when including
+#'   long fixed vectors of population coefficients, including data
+#'   which should be used across all conditions and replications (e.g., including a fixed design matrix
+#'   for linear regression), or simply to control global elements such as sample size
+#'
 #' @param parallel logical; use parallel processing from the \code{parallel} package over each
 #'   unique condition?
 #'
@@ -99,18 +109,27 @@
 #'   \code{::} operator to locate the public functions that are not visible in the R session (e.g.,
 #'   \code{psych::describe()})
 #'
-#' @param save_every a number indicating how often to temporarily save your simulation results to
-#'   disk. Default is 1 to save after every condition is complete, but set to NA if you don't
-#'   want to save any temp files
+#' @param save_results logical; save the results returned from \code{\link{analyse}} to external .rds files
+#'   located in a 'SimDesign_results' directory/folder? If a 'SimDesign_results' folder does not exist
+#'   in the current working directory then one will be created automatically.
+#'   Use this if you would like to keep track of the individual parameters returned from the analyses.
+#'   Default is FALSE
+#'
+#' @param try_errors logical; include information about which error how often they occurred from
+#'   \code{try()} chunks or \code{\link{check_error}}? If TRUE, this information will be stacked at the end
+#'   of the returned simulation results with the name of the specific error used as the column name in the
+#'   data.frame object, and the number of occurrences included as the value for each condition
 #'
 #' @param ncores number of cores to be used in parallel execution. Default uses all available
 #'
-#' @param clean logical; remove any temp files that are created after the simulation is complete?
-#'   Default is TRUE
-#'
-#' @param filename the name of the file to save the results to. Default is the system name with
+#' @param filename the name of the .rds file to save the final simulation results to.
+#'   Default is the system name with
 #'   the number of replications and 'Final' appended to the string
 #'
+#' @param results_filename the general name of the .rds file to save individual simulation results
+#'   to (before calling the \code{\link{summarise}} function). Default is the system name with '_results_'
+#'   and the row ID information appended
+#'
 #' @param tmpfilename the name of the temporary file, default is the system name with 'tmpsim.rds'
 #'   appended at the end. This file will be
 #'   read in if it is in the working directory, and the simulation will continue where at the last
@@ -128,7 +147,8 @@
 #'   may be resumed on another computer by changing the name of the node to match the broken computer
 #'
 #' @param edit a string indicating where to initiate a \code{browser()} call for editing and debugging.
-#'   Options are \code{'none'} (default), 'main' to edit the main function calls loop, \code{'generate'}
+#'   General options are \code{'none'} (default) and \code{'recover'} to disable debugging or to use the
+#'   \code{options(error = 'recover')}. Specific options include: \code{'generate'}
 #'   to edit the data simulation function, \code{'analyse'} to edit the computational function, and
 #'   \code{'summarise'} to  edit the aggregation function. Alternatively, users may place
 #'   \code{\link{browser}} calls within the respective functions for debugging at specific lines
@@ -172,7 +192,7 @@
 #' SimDesign_functions()
 #'
 #' # help(generate)
-#' Generate <- function(condition){
+#' Generate <- function(condition, fixed_design_elements = NULL){
 #'
 #'     #require packages/define functions if needed, or better yet index with the :: operator
 #'
@@ -180,16 +200,13 @@
 #'     grs <- condition$group_size_ratio
 #'     sd <- condition$standard_deviation_ratio
 #'
-#'     if(grs == 0.5){
-#'         N2 <- N/3
+#'     if(grs < 1){
+#'         N2 <- N / (1/grs + 1)
 #'         N1 <- N - N2
-#'     } else if(grs == 2){
-#'         N1 <- N/3
-#'         N2 <- N - N1
 #'     } else {
-#'         N1 <- N2 <- N / 2
+#'         N1 <- N / (grs + 1)
+#'         N2 <- N - N1
 #'     }
-#'
 #'     group1 <- rnorm(N1)
 #'     group2 <- rnorm(N2, sd=sd)
 #'     dat <- data.frame(group = c(rep('g1', N1), rep('g2', N2)), DV = c(group1, group2))
@@ -199,7 +216,7 @@
 #'
 #' # help(analyse)
 #'
-#' Analyse <- function(condition, dat, parameters = NULL){
+#' Analyse <- function(condition, dat, fixed_design_elements = NULL, parameters = NULL){
 #'
 #'     # require packages/define functions if needed, or better yet index with the :: operator
 #'     require(stats)
@@ -209,7 +226,7 @@
 #'     welch <- try(t.test(DV ~ group, dat), silent=TRUE)
 #'     ind <- try(t.test(DV ~ group, dat, var.equal=TRUE), silent=TRUE)
 #'
-#'     # check if any errors occured. This will re-draw the data
+#'     # check if any errors occurred. This will re-draw the data
 #'     check_error(welch, ind)
 #'
 #'     # In this function the p values for the t-tests are returned,
@@ -221,11 +238,10 @@
 #'
 #' # help(summarise)
 #'
-#' Summarise <- function(condition, results, parameters_list = NULL){
+#' Summarise <- function(condition, results, fixed_design_elements = NULL, parameters_list = NULL){
 #'
 #'     #find results of interest here (e.g., alpha < .1, .05, .01)
-#'     nms <- c('welch', 'independent')
-#'     lessthan.05 <- EDR(results[,nms], alpha = .05)
+#'     lessthan.05 <- EDR(results, alpha = .05)
 #'
 #'     # return the results that will be appended to the design input
 #'     ret <- c(lessthan.05=lessthan.05)
@@ -237,7 +253,7 @@
 #' #### Step 3 --- Collect results by looping over the rows in design
 #'
 #' # test to see if it works and for debugging
-#' Final <- runSimulation(design=Design, replications=10, parallel=FALSE,
+#' Final <- runSimulation(design=Design, replications=5, parallel=FALSE,
 #'                        generate=Generate, analyse=Analyse, summarise=Summarise)
 #'
 #' \dontrun{
@@ -330,31 +346,31 @@
 #' }
 #'
 runSimulation <- function(design, replications, generate, analyse, summarise,
-                          parallel = FALSE, MPI = FALSE,
-                          save = FALSE, save_every = 1, clean = TRUE, seed = NULL,
-                          compname = Sys.info()['nodename'],
-                          filename = paste0(compname,'_Final_', replications, '.rds'),
-                          tmpfilename = paste0(compname, '_tmpsim.rds'), main = NULL,
+                          fixed_design_elements = NULL, parallel = FALSE, MPI = FALSE,
+                          try_errors = TRUE, save = FALSE, save_results = FALSE,
+                          seed = NULL, compname = Sys.info()['nodename'],
+                          filename = paste0(compname,'_Final_', replications),
+                          results_filename = paste0(compname, '_results_'),
+                          tmpfilename = paste0(compname, '_tmpsim.rds'),
                           ncores = parallel::detectCores(), edit = 'none', verbose = TRUE)
 {
+    save_every <- 1L
+    filename <- paste0(filename, '.rds')
     stopifnot(!missing(generate) || !missing(analyse) || !missing(summarise))
-    Functions <- list(generate=generate, analyse=analyse, summarise=summarise, main=main)
+    Functions <- list(generate=generate, analyse=analyse, summarise=summarise)
     stopifnot(!missing(design))
     stopifnot(!missing(replications))
     if(!is.null(seed)){
         if(length(seed) == 1L) seed <- rep(seed, nrow(design))
         stopifnot(nrow(design) == length(seed))
     }
-    FunNames <- names(Functions)
     edit <- tolower(edit)
-    if(is.null(main)) Functions$main <- SimDesign::main
     for(i in names(Functions)){
         fms <- names(formals(Functions[[i]]))
         truefms <- switch(i,
-                          main = c('index', 'condition', 'generate', 'analyse'),
-                          generate  = c('condition'),
-                          analyse = c('dat', 'parameters', 'condition'),
-                          summarise = c('results', 'parameters_list', 'condition'))
+                          generate  = c('condition', 'fixed_design_elements'),
+                          analyse = c('dat', 'parameters', 'condition', 'fixed_design_elements'),
+                          summarise = c('results', 'parameters_list', 'condition', 'fixed_design_elements'))
         if(!all(truefms %in% fms))
             stop(paste0('Function arguments for ', i, ' are not correct.'), call. = FALSE)
     }
@@ -377,15 +393,21 @@ runSimulation <- function(design, replications, generate, analyse, summarise,
     if(!is.na(save_every))
         if(save_every > nrow(design))
             warning('save_every is too large to be useful', call. = FALSE)
-    if(!(edit %in% c('none', 'analyse', 'generate', 'main', 'summarise')))
+    if(!(edit %in% c('none', 'recover', 'analyse', 'generate', 'summarise')))
         stop('edit location is not valid', call. = FALSE)
     if(is.null(design$ID)){
         design <- data.frame(ID=1L:nrow(design), design)
     } else stopifnot(length(unique(design$ID)) == nrow(design))
     if(edit != 'none'){
         parallel <- MPI <- FALSE
-        debug(Functions[[edit]])
-        on.exit(undebug(Functions[[edit]]))
+        if(edit == 'recover'){
+            old_recover <- getOption('error')
+            options(error = utils::recover)
+            on.exit(options(error = old_recover))
+        } else {
+            debug(Functions[[edit]])
+            on.exit(undebug(Functions[[edit]]))
+        }
     }
     cl <- NULL
     if(parallel){
@@ -403,27 +425,37 @@ runSimulation <- function(design, replications, generate, analyse, summarise,
         Result_list <- readRDS(tmpfilename)
         start <- min(which(sapply(Result_list, is.null)))
     }
+    if(save_results)
+        dir.create('SimDesign_results', showWarnings = FALSE)
     for(i in start:nrow(design)){
         stored_time <- do.call(c, lapply(Result_list, function(x) x$SIM_TIME))
         if(verbose)
             cat(sprintf('\rCompleted: %i%s,   Previous cell time: %.1f,  Total elapsed time: %.1f ',
                         round((i-1)/(nrow(design))*100), '%', time1 - time0, sum(stored_time)))
         time0 <- proc.time()[3]
-        Result_list[[i]] <- as.data.frame(c(as.list(design[i, ]),
-                                            as.list(Analysis(Functions=Functions,
-                                                             condition=design[i,],
-                                                             replications=replications,
-                                                             cl=cl, MPI=MPI, seed=seed))))
+        Result_list[[i]] <- data.frame(c(as.list(design[i, ]),
+                                         as.list(Analysis(Functions=Functions,
+                                                          condition=design[i,],
+                                                          replications=replications,
+                                                          fixed_design_elements=fixed_design_elements,
+                                                          cl=cl, MPI=MPI, seed=seed,
+                                                          save_results=save_results,
+                                                          results_filename=results_filename))),
+                                       check.names=FALSE)
         time1 <- proc.time()[3]
         Result_list[[i]]$SIM_TIME <- time1 - time0
-        if(!(length(unique(sapply(Result_list, length))) %in% c(1L, 2L)))
-            stop(c('Summerise() results are not all of the same length. This may require splitting up',
-                   '\nthe design input to allow for different result lengths.'), call.=FALSE)
         if(save && !is.na(save_every))
             if((i %% save_every) == 0L) saveRDS(Result_list, tmpfilename)
     }
-    Final <- do.call(rbind, Result_list)
-    Final$ID <- NULL
+    Final <- plyr::rbind.fill(Result_list)
+    SIM_TIME <- Final$SIM_TIME
+    Final$SIM_TIME <- Final$ID <- NULL
+    pick <- grepl('TRY_ERROR_MESSAGE', names(Final))
+    TRY_ERRORS <- Final[,pick, drop=FALSE]
+    Final <- Final[,!pick, drop=FALSE]
+    Final <- if(try_errors){
+        data.frame(Final, REPLICATIONS=replications, SIM_TIME, TRY_ERRORS, check.names=FALSE)
+    } else data.frame(Final, REPLICATIONS=replications, SIM_TIME, check.names=FALSE)
     #save file
     files <- dir()
     filename0 <- filename
@@ -442,8 +474,8 @@ runSimulation <- function(design, replications, generate, analyse, summarise,
     if(save){
         if(verbose)
             message(paste('\nSaving simulation results to file:', filename))
+        file.remove(tmpfilename)
         saveRDS(Final, filename)
     }
-    if(clean) system(paste0('rm -f ', tmpfilename))
     return(invisible(Final))
 }
